/* Embedded OS Loader
 *
 *   Copyright 2005 Vivien Chappelier <vivien.chappelier@free.fr>
 *
 * low-level disk access
 *
 */

#include "stdlib.h"
#include "disk.h"

/* decode C/H/S address */
void chs_decode(chs_t *chs, unsigned char *addr)
{
  chs->h = addr[0];
  chs->s = addr[1] & 0x3f;
  chs->c = ((addr[1] & 0xc0) << 2) | addr[2];
}

/* read a partition table entry */
void read_pentry(unsigned char *buffer, pentry_t *pentry)
{
  pentry->boot = buffer[0];
  chs_decode(&pentry->begin, &buffer[1]);
  pentry->type = buffer[4];
  chs_decode(&pentry->end, &buffer[5]);
  pentry->sector_start   = buffer[ 8];
  pentry->sector_start  |= buffer[ 9] <<  8;
  pentry->sector_start  |= buffer[10] << 16;
  pentry->sector_start  |= buffer[11] << 24;
  pentry->sector_length  = buffer[12];
  pentry->sector_length |= buffer[13] <<  8;
  pentry->sector_length |= buffer[14] << 16;
  pentry->sector_length |= buffer[15] << 24;
}

/* read the partition table from the bootsector */
int read_ptable(unsigned char bootsector[], pentry_t ptable[4])
{
  /* check for the signature at offset 510 */
  if(bootsector[510] != 0x55 || bootsector[511] != 0xaa) {
    printb("read_ptable: no signature\n");
    return(0);
  }

  /* partition descriptors are at offset 446, 462, 478 and 494 */
  read_pentry(&bootsector[446], &ptable[0]);
  read_pentry(&bootsector[462], &ptable[1]);
  read_pentry(&bootsector[478], &ptable[2]);
  read_pentry(&bootsector[494], &ptable[3]);

  return(1);
}

/* print a partition entry */
void print_pentry(pentry_t *pentry)
{
  printb("%c ", pentry->boot ? 'B' : '-');
  printb("%x ", pentry->type);

  printb("%p ", (void *) (pentry->sector_start * 512));
  printb("%p ", (void *) (pentry->sector_length * 512));

  /*  printb("%x/%x/%x ", pentry->begin.c, pentry->begin.h, pentry->begin.s); */
  /*  printb("%x/%x/%x ", pentry->begin.c, pentry->begin.h, pentry->begin.s); */
  printb("\n");
}
