/* merge a disk image with a bootloader to create a bootable disk image */
/* the resulting image structure is as follows:
     0x000 - 0x1b0       : bootloader header (boot.img)
     0x1b0 - 0x200       : partition table   (disk.img)
     0x200 - boot.end    : bootloader code   (boot.img)
     boot.end - disk.end : disk data         (disk.img) */

#include <stdio.h>

int main(int argc, char *argv[])
{
  char *diskname, *bootname;
  FILE *disk, *boot;
  int length;
  char buffer[0x200];

  if(argc != 3) {
    fprintf(stderr, "usage: %s disk.img boot.img\n", argv[0]);
    fprintf(stderr, "\tmakes disk.img bootable using boot.img\n");
    return(1);
  }
  
  diskname = argv[1];
  bootname = argv[2];

  /* open files */
  if((disk = fopen(diskname, "r+b")) == NULL) {
    fprintf(stderr, "could not open %s in read/write mode: ", diskname);
    perror("");
    return(2);
  }

  if((boot = fopen(bootname, "rb")) == NULL) {
    fprintf(stderr, "could not open %s in read mode: ", bootname);
    perror("");
    return(3);
  }
  
  /* copy boot image header to disk */
  fread(buffer, 0x200, 1, disk);
  fread(buffer, 0x1b0, 1, boot);
  fseek(disk, 0, SEEK_SET);
  fwrite(buffer, 0x200, 1, disk);
  fread(buffer, 0x200 - 0x1b0, 1, boot);

  /* copy boot image content to disk */
  while((length = fread(buffer, 1, 0x200, boot)) > 0)
    fwrite(buffer, length, 1, disk);

  /* close files */
  fclose(disk);
  fclose(boot);

  return(0);
}
